/*
	RawToWavCmd.cc

	Will Eighteen, 2000: will@opaline.demon.co.uk
*/                                                                             

#include "RawToWavCmd.h"
#include "OpResources.h"
#include <FileAccess.h>
#include "wave/waveclass.h"
#include <MessageDialogManager.h>


RawToWavCmd::RawToWavCmd ( char *name, int active, char *rawFile, char *wavFile ) : BreakCmd ( name, active )
{
	strncpy ( _srcFile, rawFile, BUFSIZE );
	_srcFile [ sizeof ( _srcFile ) - 1 ] = 0;
	strncpy ( _dstFile, wavFile, BUFSIZE );
	_dstFile [ sizeof ( _dstFile ) - 1 ] = 0;
	_once = 0;
	_blockLength = mixwaveResources->blockLength();
	_totalSamples = 0;
	_pWave = NULL;
	_percentDone = 0;
	_error = 0;
}


RawToWavCmd::~RawToWavCmd()
{	
	// empty
}
	

void RawToWavCmd::doCommand()
{
	int percent;
	char text[5];
	int error = 0;

	if ( _done == True )
		return;
	
	if ( !_once )
	{
		// can we open source for read?
		FileAccess *src = new FileAccess ( _srcFile, "r" );
		
		if ( src->errNum() != 0 )
		{
			char msg[BUFSIZE*2];
			
			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nFile read error: %s", _srcFile, src->error() );
			msg [ sizeof (msg ) - 1 ] = 0;			
			messageDialogManager->post ( msg );
		}
		
		delete src;
		
		// can we open dst for write?
		FileAccess *dst = new FileAccess ( _dstFile, "w" );
		
		if ( dst->errNum() != 0 )
		{
			char msg[BUFSIZE*2];
			
			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nmFile write error: %s", _dstFile, dst->error() );
			msg [ sizeof (msg ) - 1 ] = 0;			
			messageDialogManager->post ( msg );
		}
		
		delete dst;

		if ( error )
		{
			_done = True;
			_error = error;
			return;
		}

		_pWave = new Wave;
		
		_pWave->FillHeader ( _channels, _samplesPerSec, _bitsPerSample, 0 );

		if ( !_pWave->SaveHeader ( _dstFile ) )
		{
			char msg[BUFSIZE];
	  
	   		strncpy ( msg, _pWave->name(), sizeof ( msg ) );
			msg [ sizeof ( msg ) - 1 ] = 0;
			strncat ( msg, "\nWave::SaveHeader failed.", sizeof ( msg ) - strlen ( msg ) - 1 );
			messageDialogManager->post ( msg );
			delete _pWave;
			_done = True;
			_error++;
			return;
		}
		
		FileAccess *in = new FileAccess ( _srcFile, "r" );
		FileAccess *out = new FileAccess ( _dstFile, "w"  );
		
		_in = in;
		_out = out;
		
		if ( _blockLength == 0 )
			_blockLength = in->fileSize() - 1;
			
		_out->filePos ( _pWave->HeaderSize() );	
		updateTimeout ( ( int ) ( _in->fileSize() / mixwaveResources->blockLength() ) + 1 );
		_once = 1;
	}
	
	if ( _blockLength > ( _in->fileSize() - _in->filePos() ) )
		_blockLength =  _in->fileSize() - _in->filePos();
	else
		if ( mixwaveResources->blockLength() > 0 )
			_blockLength = mixwaveResources->blockLength();

	char buf[_blockLength];
	
	if ( _in->filePos() < _in->fileSize() )
	{
		_in->readF ( &buf, _blockLength, 1 );
		_out->writeF ( &buf, _blockLength, 1 );
		_totalSamples += _blockLength>>( ( _bitsPerSample / 8 ) - 1 );
		_pWave->FillHeader ( _channels, _samplesPerSec, _bitsPerSample, _totalSamples / _channels );
		_pWave->ChangeHeader ( _dstFile );

		percent = ( int ) ( ( float ) _in->filePos() / ( float ) _in->fileSize() * 100 );

		if ( _percentDone != percent )
		{
			_percentDone = percent;
			snprintf ( text, 5, "%d%%", _percentDone );
			updateMessage ( text );
		}
	}

	if ( _in->filePos() == _in->fileSize() )
	{
		delete _in;
		delete _out;
		delete _pWave;
		_done = True;
	}
}


void RawToWavCmd::setWaveDefaults ( long samplesPerSec, short bitsPerSample, short channels )
{
	_samplesPerSec = samplesPerSec;
	_bitsPerSample = bitsPerSample;
	_channels = channels;
}
