/*
    SelectFilesCmd.cc

    Will 18, 2001: will@opaline.demon.co.uk
*/


#include "SelectFilesCmd.h"
#include <FileSelect.h>
#include <MessageDialogManager.h>
#include <FileAccess.h>
#include "OpResources.h"


SelectFilesCmd::SelectFilesCmd ( char *name, int active, SelectFilesCallback callback,
									char *title1, char *title2, Boolean overwrite, void *clientData ) : NoUndoCmd ( name, active )
{
	_callback = callback;
	_clientData = clientData;
	_name = name;
	_selectFiles = False;
	_title1 = title1;
	_title2 = title2;
	_overwrite = overwrite;	// check 2nd file exists if true
}


void SelectFilesCmd::doCommand()
{
	if ( _selectFiles == True )
		return;
	else
		_selectFiles = True;

	_srcFileName[0] = 0;
	_dstFileName[0] = 0;
	
	char srcTitle[BUFSIZE];
	char dstTitle[BUFSIZE];

	snprintf ( srcTitle, sizeof ( srcTitle ), "%s %s", _name, _title1 );
	srcTitle [ sizeof ( srcTitle ) - 1 ] = 0;
	snprintf ( dstTitle, sizeof ( dstTitle ), "%s %s", _name, _title2 );
	dstTitle [ sizeof ( dstTitle ) - 1 ] = 0;

	_dst = new FileSelect ( dstTitle, TRUE,
				&SelectFilesCmd::targetSelectedCallback,
				&SelectFilesCmd::fileSelectCancelledCallback,
				&SelectFilesCmd::targetCancelledCallback, ( void * ) this );

	_src = new FileSelect ( srcTitle, TRUE,
				&SelectFilesCmd::sourceSelectedCallback,
				&SelectFilesCmd::fileSelectCancelledCallback,
				&SelectFilesCmd::sourceCancelledCallback, ( void * ) this );

	_src->execute();
}


void SelectFilesCmd::sourceSelectedCallback ( void *clientData, char *filename )
{
	SelectFilesCmd *obj = ( SelectFilesCmd * ) clientData;

	if ( filename[strlen(filename) - 1] != '/' )	// no file specified
		obj->sourceSelected ( filename );
		
	delete obj->_src;
	obj->_selectFiles = False;	
}


void SelectFilesCmd::fileSelectCancelledCallback ( void *clientData )
{
	// empty
}


void SelectFilesCmd::sourceSelected ( char *filename )
{
	strncpy ( _srcFileName, filename, sizeof ( _srcFileName ) );
	_srcFileName [ sizeof ( _srcFileName ) - 1 ] = 0;
	// don't allow callback - dealt with by dst browser
	_dst->execute();
}


void SelectFilesCmd::sourceCancelledCallback ( void *clientData )
{
	SelectFilesCmd *obj = ( SelectFilesCmd * ) clientData;

	obj->_selectFiles = False;
	delete obj->_src;
}


void SelectFilesCmd::targetSelectedCallback ( void *clientData, char *filename )
{
	SelectFilesCmd *obj = ( SelectFilesCmd * ) clientData;

	strncpy ( obj->_dstFileName, filename, sizeof ( obj->_dstFileName ) );
	obj->_dstFileName [ sizeof ( obj->_dstFileName ) - 1 ] = 0;

	if ( obj->_overwrite == True )	// check for existing file
	{
		if ( filename[strlen(filename) - 1] != '/' )	// no file specified
			obj->isDstFile ( filename );
	}
	else
		obj->targetSelected();	// allow null filename for 2nd file

	delete obj->_dst;
	obj->_selectFiles = False;	
}


void SelectFilesCmd::targetSelected()
{
	if ( _callback )
		if ( strlen ( _srcFileName ) != 0 )
			_callback ( _clientData, _srcFileName, _dstFileName );

	_selectFiles = False;
}


void SelectFilesCmd::targetCancelledCallback ( void *clientData )
{
	SelectFilesCmd *obj = ( SelectFilesCmd * ) clientData;

	obj->_dstFileName[0] = 0;
	obj->targetSelected();	// do callback with null filename or we don't get source

	delete obj->_dst;
}


void SelectFilesCmd::isDstFile ( char * filename )
{	
	if ( mixwaveResources->enableWarningPopups() == True )
	{
		int error;
		
		FileAccess *target = new FileAccess ( filename, "r" );
		error = target->errNum();	
		delete target;
	
		if ( error == 0 )	// succeeded, file already exists
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nFile exists: overwrite?", filename );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg, ( void * ) this,
							&SelectFilesCmd::overwriteOkCallback,
							&SelectFilesCmd::overwriteCancelCallback );
		}
		else
			targetSelected();
	}
	else
		targetSelected();
}


void SelectFilesCmd::overwriteOkCallback ( void *clientData )
{
	SelectFilesCmd *obj = ( SelectFilesCmd * ) clientData;
	
	obj->targetSelected();
}


void SelectFilesCmd::overwriteCancelCallback ( void *clientData )
{
	SelectFilesCmd *obj = ( SelectFilesCmd * ) clientData;
	
	obj->_dstFileName[0] = 0;
	obj->targetSelected();
}

